import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import _extends from "@babel/runtime/helpers/esm/extends";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
var _excluded = ["events"];
import _regeneratorRuntime from "@babel/runtime/regenerator";
import createEventStoreLite from '@resolve-js/eventstore-lite';
import { prepareEvents, stripEvents } from './events-utils';
import { ambiguousEventsTimeErrorMessage, getReservedFieldUsedErrorMessage, reservedEventOrderField } from '../constants';
export var validateEvents = function validateEvents(events) {
  var stampedCount = events.filter(function (event) {
    return event.timestamp != null;
  }).length;

  if (stampedCount !== events.length && stampedCount > 0) {
    throw Error(ambiguousEventsTimeErrorMessage);
  }

  var invalidEventIndex = events.findIndex(function (event) {
    var _event$payload;

    return ((_event$payload = event.payload) == null ? void 0 : _event$payload[reservedEventOrderField]) != null;
  });

  if (invalidEventIndex >= 0) {
    throw Error(getReservedFieldUsedErrorMessage(reservedEventOrderField));
  }
};
export var getEventStore = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(events, context) {
    var preparedEvents, eventByAggregateIdVersion, eventIndex, _preparedEvents$event, aggregateId, aggregateVersion, event, eventStoreAdapter;

    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            validateEvents(events);
            preparedEvents = prepareEvents(events, context);
            eventByAggregateIdVersion = new Map();

            for (eventIndex = 0; eventIndex < events.length; eventIndex++) {
              _preparedEvents$event = preparedEvents[eventIndex], aggregateId = _preparedEvents$event.aggregateId, aggregateVersion = _preparedEvents$event.aggregateVersion;
              event = events[eventIndex];

              if (event != null) {
                eventByAggregateIdVersion.set(aggregateId + ":" + aggregateVersion, event);
              }
            }

            eventStoreAdapter = createEventStoreLite({
              databaseFile: ':memory:'
            });
            _context2.next = 7;
            return eventStoreAdapter.init();

          case 7:
            _context2.next = 9;
            return Promise.all(preparedEvents.map(function (event) {
              return eventStoreAdapter.saveEvent(event);
            }));

          case 9:
            return _context2.abrupt("return", _extends({}, eventStoreAdapter, {
              loadEvents: function () {
                var _loadEvents = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
                  var _yield$eventStoreAdap,
                      storedEvents,
                      other,
                      events,
                      timestampIndex,
                      _eventByAggregateIdVe,
                      _eventByAggregateIdVe2,
                      _event,
                      _args = arguments;

                  return _regeneratorRuntime.wrap(function _callee$(_context) {
                    while (1) {
                      switch (_context.prev = _context.next) {
                        case 0:
                          _context.next = 2;
                          return eventStoreAdapter.loadEvents.apply(eventStoreAdapter, _args);

                        case 2:
                          _yield$eventStoreAdap = _context.sent;
                          storedEvents = _yield$eventStoreAdap.events;
                          other = _objectWithoutPropertiesLoose(_yield$eventStoreAdap, _excluded);
                          events = stripEvents(JSON.parse(JSON.stringify(storedEvents)));

                          for (timestampIndex = 0; timestampIndex < events.length; timestampIndex++) {
                            _event = events[timestampIndex];
                            _event.timestamp = ((_eventByAggregateIdVe = (_eventByAggregateIdVe2 = eventByAggregateIdVersion.get(_event.aggregateId + ":" + _event.aggregateVersion)) == null ? void 0 : _eventByAggregateIdVe2.timestamp) !== null && _eventByAggregateIdVe !== void 0 ? _eventByAggregateIdVe : timestampIndex) + 1;
                          }

                          return _context.abrupt("return", _extends({}, other, {
                            events: events
                          }));

                        case 8:
                        case "end":
                          return _context.stop();
                      }
                    }
                  }, _callee);
                }));

                function loadEvents() {
                  return _loadEvents.apply(this, arguments);
                }

                return loadEvents;
              }()
            }));

          case 10:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function getEventStore(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();
//# sourceMappingURL=get-event-store.js.map